﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;

using DevExpress.Xpo;
using DevExpress.Xpo.DB;

using Sturm.XPOUtilities.Server;
using Sturm.XPOUtilities.Contracts;
using Types;
using DevExpress.Data.Filtering;

namespace DataService {
  class Program {
    static void Main(string[] args) {
      // Create the real data store - any DB backend would do, of course
      IDataStore sourceDataStore = XpoDefault.GetConnectionProvider(AutoCreateOption.DatabaseAndSchema);

      // Put some demo data in, so we have something to see
      InitDemoData(sourceDataStore);

      // Wrap the data store in a console logger
      sourceDataStore = new DataStoreLogger(sourceDataStore, Console.Out);

      // Wrap it in a cache root object
      var cacheRoot = new DataCacheRoot(sourceDataStore);

      // Wrap it in the WCF contract implementation
      var publicationObject = new CachedDataStoreProxy(cacheRoot);

      // Publish the contract via WCF. This could obviously be done with config files instead of code,
      // using different bindings and endpoints, whatever...

      var baseAddress = new Uri("net.tcp://0.0.0.0:7465/");
      var serviceHost = new ServiceHost(publicationObject, baseAddress);
      var binding = new NetTcpBinding(SecurityMode.None);
      serviceHost.AddServiceEndpoint(typeof(ICacheToCacheCommunicationCoreContract), binding, "XPOService");
      serviceHost.Open( );

      Console.WriteLine("Service published, hit Return to terminate.");
      Console.ReadLine( );

      serviceHost.Close( );
    }

    private static void InitDemoData(IDataStore sourceDataStore) {
      var dataLayer = new SimpleDataLayer(sourceDataStore);
      using (var uow = new UnitOfWork(dataLayer)) {
        if (uow.FindObject<Product>(new BinaryOperator("Name", "Rubber Chicken")) == null) {
          var rubberChicken = new Product(uow) { Name = "Rubber Chicken" };
          var pulley = new Product(uow) { Name = "Pulley" };
          var bread = new Product(uow) { Name = "Bread" };
          var butter = new Product(uow) { Name = "Butter" };

          new OrderLine(uow) { Count = 18, Product = rubberChicken };
          new OrderLine(uow) { Count = 18, Product = pulley };
          new OrderLine(uow) { Count = 4, Product = rubberChicken };
          new OrderLine(uow) { Count = 8, Product = pulley };
          new OrderLine(uow) { Count = 100, Product = bread };
          new OrderLine(uow) { Count = 100, Product = butter };

          uow.CommitChanges( );
        }
      }
    }
  }
}
