﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DevExpress.Xpo;
using DevExpress.Data.Filtering;
using DevExpress.Xpo.DB;

namespace XPOWebinarPersistentClasses {
  class Program {
    static void Main(string[] args) {
      //XpoDefault.DataLayer = XpoDefault.GetDataLayer(
      //  MSSqlConnectionProvider.GetConnectionString(".", "XPOWebinarPersistentClasses"),
      //  AutoCreateOption.DatabaseAndSchema);

      var connectionProvider = new DataStoreLogger(
        XpoDefault.GetConnectionProvider(
        MSSqlConnectionProvider.GetConnectionString(".", "XPOWebinarPersistentClasses"),
        AutoCreateOption.DatabaseAndSchema), Console.Out);
      XpoDefault.DataLayer = new SimpleDataLayer(connectionProvider);

      using (UnitOfWork uow = new UnitOfWork( )) {
        var rubberChicken = new Product(uow) {
          Name = "Rubber Chicken"
        };
        var pulley = new Product(uow) {
          Name = "Pulley"
        };
        var orderLine = new OrderLine(uow) {
          Count = 10,
          Product = rubberChicken
        };
        orderLine = new OrderLine(uow) {
          Count = 10,
          Product = pulley
        };
        orderLine = new OrderLine(uow) {
          Count = 4,
          Product = rubberChicken
        };
        orderLine = new OrderLine(uow) {
          Count = 8,
          Product = pulley
        };

        uow.CommitChanges( );
      }

      using (var uow = new UnitOfWork( )) {
        //var smallOrderLines = new XPCollection<OrderLine>(uow,
        //  new BinaryOperator("Count", 5, BinaryOperatorType.Less));

        var smallOrderLines =
          from line in new XPQuery<OrderLine>(uow)
          where line.Count < 5
          select line;

        foreach (var line in smallOrderLines) {
          Console.WriteLine("{0} sold {1} times", line.Product.Name, line.Count);
        }
      }
    }
  }

  public class OrderLine : XPObject {
    public OrderLine(Session session)
      : base(session) { }

    private int count;
    //[DbType("...")]
    public int Count {
      get {
        return count;
      }
      set {
        SetPropertyValue("Count", ref count, value);
      }
    }

    private Product product;
    [Association("Product-OrderLines")]
    public Product Product {
      get {
        return product;
      }
      set {
        SetPropertyValue("Product", ref product, value);
      }
    }
  }

  public class Product : XPObject {
    public Product(Session session)
      : base(session) { }

    private string name;
    public string Name {
      get {
        return name;
      }
      set {
        SetPropertyValue("Name", ref name, value);
      }
    }

    [Association("Product-OrderLines")]
    public XPCollection<OrderLine> OrderLines {
      get {
        return GetCollection<OrderLine>("OrderLines");
      }
    }
  }
}
